@@ -352,6 +352,102 @@ public class IoBufferWrapper extends IoBuffer {
         buf.putInt(value);
         return this;
     }
+    
+    @Override
+    public IoBuffer putUnsignedInt(byte value) {
+        buf.putUnsignedInt(value);
+        return this;
+    }
+    
+    @Override
+    public IoBuffer putUnsignedInt(int index, byte value) {
+        buf.putUnsignedInt(index, value);
+        return this;
+    }
+    
+    @Override
+    public IoBuffer putUnsignedInt(short value) {
+        buf.putUnsignedInt(value);
+        return this;
+    }
+    
+    @Override
+    public IoBuffer putUnsignedInt(int index, short value) {
+        buf.putUnsignedInt(index, value);
+        return this;
+    }
+    
+    @Override
+    public IoBuffer putUnsignedInt(int value) {
+        buf.putUnsignedInt(value);
+        return this;
+    }
+    
+    @Override
+    public IoBuffer putUnsignedInt(int index, int value) {
+        buf.putUnsignedInt(index, value);
+        return this;
+    }
+
+    @Override
+    public IoBuffer putUnsignedInt(long value) {
+        buf.putUnsignedInt(value);
+        return this;
+    }
+    
+    @Override
+    public IoBuffer putUnsignedInt(int index, long value) {
+        buf.putUnsignedInt(index, value);
+        return this;
+    }
+    
+    @Override
+    public IoBuffer putUnsignedShort(byte value) {
+        buf.putUnsignedShort(value);
+        return this;
+    }
+    
+    @Override
+    public IoBuffer putUnsignedShort(int index, byte value) {
+        buf.putUnsignedShort(index, value);
+        return this;
+    }
+    
+    @Override
+    public IoBuffer putUnsignedShort(short value) {
+        buf.putUnsignedShort(value);
+        return this;
+    }
+    
+    @Override
+    public IoBuffer putUnsignedShort(int index, short value) {
+        buf.putUnsignedShort(index, value);
+        return this;
+    }
+    
+    @Override
+    public IoBuffer putUnsignedShort(int value) {
+        buf.putUnsignedShort(value);
+        return this;
+    }
+    
+    @Override
+    public IoBuffer putUnsignedShort(int index, int value) {
+        buf.putUnsignedShort(index, value);
+        return this;
+    }
+
+    @Override
+    public IoBuffer putUnsignedShort(long value) {
+        buf.putUnsignedShort(value);
+        return this;
+    }
+    
+    @Override
+    public IoBuffer putUnsignedShort(int index, long value) {
+        buf.putUnsignedShort(index, value);
+        return this;
+    }
 
     @Override
     public int getInt(int index) {
@@ -897,4 +993,52 @@ public class IoBufferWrapper extends IoBuffer {
         buf.putEnumSetLong(index, set);
         return this;
     }
+
+    @Override
+    public IoBuffer putUnsigned(byte value) {
+        buf.putUnsigned(value);
+        return this;
+    }
+
+    @Override
+    public IoBuffer putUnsigned(int index, byte value) {
+        buf.putUnsigned(index, value);
+        return this;
+    }
+
+    @Override
+    public IoBuffer putUnsigned(short value) {
+        buf.putUnsigned(value);
+        return this;
+    }
+
+    @Override
+    public IoBuffer putUnsigned(int index, short value) {
+        buf.putUnsigned(index, value);
+        return this;
+    }
+
+    @Override
+    public IoBuffer putUnsigned(int value) {
+        buf.putUnsigned(value);
+        return this;
+    }
+
+    @Override
+    public IoBuffer putUnsigned(int index, int value) {
+        buf.putUnsigned(index, value);
+        return this;
+    }
+
+    @Override
+    public IoBuffer putUnsigned(long value) {
+        buf.putUnsigned(value);
+        return this;
+    }
+
+    @Override
+    public IoBuffer putUnsigned(int index, long value) {
+        buf.putUnsigned(index, value);
+        return this;
+    }
 }
