@@ -27,6 +27,7 @@ import java.util.Queue;
 import java.util.concurrent.ConcurrentLinkedQueue;
 import java.util.concurrent.Executor;
 import java.util.concurrent.Executors;
+import java.util.concurrent.atomic.AtomicReference;
 
 import org.apache.mina.core.RuntimeIoException;
 import org.apache.mina.core.filterchain.IoFilter;
@@ -63,7 +64,6 @@ import org.apache.mina.util.ExceptionMonitor;
 public abstract class AbstractPollingIoConnector<T extends AbstractIoSession, H>
         extends AbstractIoConnector {
 
-    private final Object lock = new Object();
     private final Queue<ConnectionRequest> connectQueue = new ConcurrentLinkedQueue<ConnectionRequest>();
     private final Queue<ConnectionRequest> cancelQueue = new ConcurrentLinkedQueue<ConnectionRequest>();
     private final IoProcessor<T> processor;
@@ -74,7 +74,7 @@ public abstract class AbstractPollingIoConnector<T extends AbstractIoSession, H>
     private volatile boolean selectable;
     
     /** The connector thread */
-    private Connector connector;
+    private final AtomicReference<Connector> connectorRef = new AtomicReference<Connector>();
 
     /**
      * Constructor for {@link AbstractPollingIoConnector}. You need to provide a default
@@ -353,9 +353,12 @@ public abstract class AbstractPollingIoConnector<T extends AbstractIoSession, H>
             cancelQueue.clear();
         }
 
-        synchronized (lock) {
-            if (connector == null) {
-                connector = new Connector();
+        Connector connector = connectorRef.get();
+        
+        if (connector == null) {
+            connector = new Connector();
+            
+            if (connectorRef.compareAndSet(null, connector)) {
                 executeWorker(connector);
             }
         }
@@ -387,13 +390,16 @@ public abstract class AbstractPollingIoConnector<T extends AbstractIoSession, H>
 
     private int cancelKeys() {
         int nHandles = 0;
+        
         for (; ;) {
             ConnectionRequest req = cancelQueue.poll();
+            
             if (req == null) {
                 break;
             }
 
             H handle = req.handle;
+            
             try {
                 close(handle);
             } catch (Exception e) {
@@ -402,6 +408,11 @@ public abstract class AbstractPollingIoConnector<T extends AbstractIoSession, H>
                 nHandles ++;
             }
         }
+        
+        if ( nHandles > 0 ) {
+            wakeup();
+        }
+        
         return nHandles;
     }
 
@@ -463,7 +474,10 @@ public abstract class AbstractPollingIoConnector<T extends AbstractIoSession, H>
     private class Connector implements Runnable {
 
         public void run() {
+            assert (connectorRef.get() == this);
+            
             int nHandles = 0;
+            
             while (selectable) {
                 try {
                     // the timeout for select shall be smaller of the connect
@@ -473,6 +487,23 @@ public abstract class AbstractPollingIoConnector<T extends AbstractIoSession, H>
 
                     nHandles += registerNew();
 
+                    // get a chance to get out of the connector loop, if we don't have any more handles
+                    if (nHandles == 0) {
+                        connectorRef.set(null);
+
+                        if (connectQueue.isEmpty()) {
+                            assert (connectorRef.get() != this);
+                            break;
+                        }
+                        
+                        if (!connectorRef.compareAndSet(null, this)) {
+                            assert (connectorRef.get() != this);
+                            break;
+                        }
+                        
+                        assert (connectorRef.get() == this);
+                    }
+
                     if (selected > 0) {
                         nHandles -= processConnections(selectedHandles());
                     }
@@ -480,15 +511,6 @@ public abstract class AbstractPollingIoConnector<T extends AbstractIoSession, H>
                     processTimedOutSessions(allHandles());
 
                     nHandles -= cancelKeys();
-
-                    if (nHandles == 0) {
-                        synchronized (lock) {
-                            if (connectQueue.isEmpty()) {
-                                connector = null;
-                                break;
-                            }
-                        }
-                    }
                 } catch (ClosedSelectorException cse) {
                     // If the selector has been closed, we can exit the loop
                     break;
