@@ -21,6 +21,7 @@ package org.apache.mina.core.polling;
 
 import java.io.IOException;
 import java.net.PortUnreachableException;
+import java.nio.channels.ClosedSelectorException;
 import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.List;
@@ -31,6 +32,7 @@ import java.util.concurrent.ConcurrentLinkedQueue;
 import java.util.concurrent.Executor;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
+import java.util.concurrent.atomic.AtomicReference;
 
 import org.apache.mina.core.buffer.IoBuffer;
 import org.apache.mina.core.file.FileRegion;
@@ -58,9 +60,9 @@ import org.slf4j.LoggerFactory;
  * developers to write an {@link IoProcessor} easily. This class is in charge of
  * active polling a set of {@link IoSession} and trigger events when some I/O
  * operation is possible.
- * 
+ *
  * @author <a href="http://mina.apache.org">Apache MINA Project</a>
- * 
+ *
  * @param <S> the type of the {@link IoSession} this processor can handle
  */
 public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> implements IoProcessor<S> {
@@ -84,9 +86,6 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
     /** A map containing the last Thread ID for each class */
     private static final Map<Class<?>, AtomicInteger> threadIds = new ConcurrentHashMap<Class<?>, AtomicInteger>();
 
-    /** A lock used to protect the processor creation */
-    private final Object lock = new Object();
-
     /** This IoProcessor instance name */
     private final String threadName;
 
@@ -109,7 +108,7 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
     private final Queue<S> trafficControllingSessions = new ConcurrentLinkedQueue<S>();
 
     /** The processor thread : it handles the incoming messages */
-    private Processor processor;
+    private final AtomicReference<Processor> processorRef = new AtomicReference<Processor>();
 
     private long lastIdleCheckTime;
 
@@ -126,7 +125,7 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
     /**
      * Create an {@link AbstractPollingIoProcessor} with the given
      * {@link Executor} for handling I/Os events.
-     * 
+     *
      * @param executor
      *            the {@link Executor} for handling I/O events
      */
@@ -143,7 +142,7 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
      * Compute the thread ID for this class instance. As we may have different
      * classes, we store the last ID number into a Map associating the class
      * name to the last assigned ID.
-     * 
+     *
      * @return a name for the current thread, based on the class name and an
      *         incremental value, starting at 1.
      */
@@ -208,14 +207,14 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
     /**
      * Dispose the resources used by this {@link IoProcessor} for polling the
      * client connections. The implementing class doDispose method will be called.
-     * 
+     *
      * @throws Exception if some low level IO error occurs
      */
     protected abstract void doDispose() throws Exception;
 
     /**
      * poll those sessions for the given timeout
-     * 
+     *
      * @param timeout
      *            milliseconds before the call timeout if no event appear
      * @return The number of session ready for read or for write
@@ -226,7 +225,7 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
 
     /**
      * poll those sessions forever
-     * 
+     *
      * @return The number of session ready for read or for write
      * @throws Exception
      *             if some low level IO error occurs
@@ -236,7 +235,7 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
     /**
      * Say if the list of {@link IoSession} polled by this {@link IoProcessor}
      * is empty
-     * 
+     *
      * @return true if at least a session is managed by this {@link IoProcessor}
      */
     protected abstract boolean isSelectorEmpty();
@@ -249,13 +248,13 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
     /**
      * Get an {@link Iterator} for the list of {@link IoSession} polled by this
      * {@link IoProcessor}
-     * 
+     *
      * @return {@link Iterator} of {@link IoSession}
      */
     protected abstract Iterator<S> allSessions();
 
     /**
-     * Get an {@link Iterator} for the list of {@link IoSession} found selected 
+     * Get an {@link Iterator} for the list of {@link IoSession} found selected
      * by the last call of {@link AbstractPollingIoProcessor#select(int)
      * @return {@link Iterator} of {@link IoSession} read for I/Os operation
      */
@@ -263,7 +262,7 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
 
     /**
      * Get the state of a session (preparing, open, closed)
-     * 
+     *
      * @param session
      *            the {@link IoSession} to inspect
      * @return the state of the session
@@ -272,7 +271,7 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
 
     /**
      * Is the session ready for writing
-     * 
+     *
      * @param session
      *            the session queried
      * @return true is ready, false if not ready
@@ -281,7 +280,7 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
 
     /**
      * Is the session ready for reading
-     * 
+     *
      * @param session
      *            the session queried
      * @return true is ready, false if not ready
@@ -290,7 +289,7 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
 
     /**
      * register a session for writing
-     * 
+     *
      * @param session
      *            the session registered
      * @param isInterested
@@ -301,7 +300,7 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
 
     /**
      * register a session for reading
-     * 
+     *
      * @param session
      *            the session registered
      * @param isInterested
@@ -312,7 +311,7 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
 
     /**
      * is this session registered for reading
-     * 
+     *
      * @param session
      *            the session queried
      * @return true is registered for reading
@@ -321,7 +320,7 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
 
     /**
      * is this session registered for writing
-     * 
+     *
      * @param session
      *            the session queried
      * @return true is registered for writing
@@ -330,7 +329,7 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
 
     /**
      * Initialize the polling of a session. Add it to the polling process.
-     * 
+     *
      * @param session the {@link IoSession} to add to the polling
      * @throws Exception any exception thrown by the underlying system calls
      */
@@ -338,7 +337,7 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
 
     /**
      * Destroy the underlying client socket handle
-     * 
+     *
      * @param session
      *            the {@link IoSession}
      * @throws Exception
@@ -349,7 +348,7 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
     /**
      * Reads a sequence of bytes from a {@link IoSession} into the given
      * {@link IoBuffer}. Is called when the session was found ready for reading.
-     * 
+     *
      * @param session
      *            the session to read
      * @param buf
@@ -363,7 +362,7 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
     /**
      * Write a sequence of bytes to a {@link IoSession}, means to be called when
      * a session was found ready for writing.
-     * 
+     *
      * @param session
      *            the session to write
      * @param buf
@@ -383,7 +382,7 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
      * isn't supporting system calls like sendfile(), you can throw a
      * {@link UnsupportedOperationException} so the file will be send using
      * usual {@link #write(AbstractIoSession, IoBuffer, int)} call.
-     * 
+     *
      * @param session
      *            the session to write
      * @param region
@@ -455,9 +454,12 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
      * pool. The Runnable will be renamed
      */
     private void startupProcessor() {
-        synchronized (lock) {
-            if (processor == null) {
-                processor = new Processor();
+        Processor processor = processorRef.get();
+
+        if (processor == null) {
+            processor = new Processor();
+
+            if (processorRef.compareAndSet(null, processor)) {
                 executor.execute(new NamePreservingRunnable(processor, threadName));
             }
         }
@@ -467,10 +469,31 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
         wakeup();
     }
 
+    /**
+     * In the case we are using the java select() method, this method is used to
+     * trash the buggy selector and create a new one, registring all the sockets
+     * on it.
+     *
+     * @throws IOException
+     *             If we got an exception
+     */
+    abstract protected void registerNewSelector() throws IOException;
+
+    /**
+     * Check that the select() has not exited immediately just because of a
+     * broken connection. In this case, this is a standard case, and we just
+     * have to loop.
+     *
+     * @return true if a connection has been brutally closed.
+     * @throws IOException
+     *             If we got an exception
+     */
+    abstract protected boolean isBrokenConnection() throws IOException;
+
     /**
      * Loops over the new sessions blocking queue and returns the number of
      * sessions which are effectively created
-     * 
+     *
      * @return The number of new sessions
      */
     private int handleNewSessions() {
@@ -513,7 +536,7 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
             listeners.fireSessionCreated(session);
         } catch (Throwable e) {
             ExceptionMonitor.getInstance().exceptionCaught(e);
-            
+
             try {
                 destroy(session);
             } catch (Exception e1) {
@@ -522,7 +545,7 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
                 registered = false;
             }
         }
-        
+
         return registered;
     }
 
@@ -539,29 +562,29 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
                     if (removeNow(session)) {
                         removedSessions++;
                     }
-    
+
                     break;
-    
+
                 case CLOSING:
                     // Skip if channel is already closed
                     break;
-    
+
                 case OPENING:
                     // Remove session from the newSessions queue and
                     // remove it
                     newSessions.remove(session);
-    
+
                     if (removeNow(session)) {
                         removedSessions++;
                     }
-                    
+
                     break;
-    
+
                 default:
                     throw new IllegalStateException(String.valueOf(state));
             }
         }
-        
+
         return removedSessions;
     }
 
@@ -590,7 +613,7 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
 
         if ((req = writeRequestQueue.poll(session)) != null) {
             Object message = req.getMessage();
-            
+
             if (message instanceof IoBuffer) {
                 IoBuffer buf = (IoBuffer)message;
 
@@ -617,12 +640,12 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
         if (!failedRequests.isEmpty()) {
             WriteToClosedSessionException cause = new WriteToClosedSessionException(
                     failedRequests);
-            
+
             for (WriteRequest r : failedRequests) {
                 session.decreaseScheduledBytesAndMessages(r);
                 r.getFuture().setException(cause);
             }
-            
+
             IoFilterChain filterChain = session.getFilterChain();
             filterChain.fireExceptionCaught(cause);
         }
@@ -650,7 +673,7 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
             // add the session to the queue, if it's not already there
             if (session.setScheduledForFlush(true)) {
                 flushingSessions.add(session);
-            }       
+            }
         }
     }
 
@@ -668,17 +691,17 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
 
             try {
                 if (hasFragmentation) {
-                    
+
                     while ((ret = read(session, buf)) > 0) {
                         readBytes += ret;
-                        
+
                         if (!buf.hasRemaining()) {
                             break;
                         }
                     }
                 } else {
                     ret = read(session, buf);
-                    
+
                     if (ret > 0) {
                         readBytes = ret;
                     }
@@ -758,7 +781,7 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
 
         do {
             S session = flushingSessions.poll(); // the same one with firstSession
-            
+
             if (session == null) {
                 // Just in case ... It should not happen.
                 break;
@@ -767,14 +790,14 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
             // Reset the Schedule for flush flag for this session,
             // as we are flushing it now
             session.unscheduledForFlush();
-            
+
             SessionState state = getState(session);
 
             switch (state) {
                 case OPENED:
                     try {
                         boolean flushedAll = flushNow(session, currentTime);
-                        
+
                         if (flushedAll
                                 && !session.getWriteRequestQueue().isEmpty(session)
                                 && !session.isScheduledForFlush()) {
@@ -785,20 +808,20 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
                         IoFilterChain filterChain = session.getFilterChain();
                         filterChain.fireExceptionCaught(e);
                     }
-    
+
                     break;
-    
+
                 case CLOSING:
                     // Skip if the channel is already closed.
                     break;
-    
+
                 case OPENING:
                     // Retry later if session is not yet fully initialized.
                     // (In case that Session.write() is called before addSession()
                     // is processed)
                     scheduleFlush(session);
                     return;
-    
+
                 default:
                     throw new IllegalStateException(String.valueOf(state));
             }
@@ -825,34 +848,34 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
                 + (session.getConfig().getMaxReadBufferSize() >>> 1);
         int writtenBytes = 0;
         WriteRequest req = null;
-        
+
         try {
             // Clear OP_WRITE
             setInterestedInWrite(session, false);
-            
+
             do {
                 // Check for pending writes.
                 req = session.getCurrentWriteRequest();
-                
+
                 if (req == null) {
                     req = writeRequestQueue.poll(session);
-                    
+
                     if (req == null) {
                         break;
                     }
-                    
+
                     session.setCurrentWriteRequest(req);
                 }
 
                 int localWrittenBytes = 0;
                 Object message = req.getMessage();
-                
+
                 if (message instanceof IoBuffer) {
                     localWrittenBytes = writeBuffer(session, req,
                             hasFragmentation, maxWrittenBytes - writtenBytes,
                             currentTime);
-                    
-                    if (localWrittenBytes > 0
+
+                    if (( localWrittenBytes > 0 )
                             && ((IoBuffer) message).hasRemaining()) {
                         // the buffer isn't empty, we re-interest it in writing
                         writtenBytes += localWrittenBytes;
@@ -869,8 +892,8 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
                     // If there's still data to be written in the FileRegion,
                     // return 0 indicating that we need
                     // to pause until writing may resume.
-                    if (localWrittenBytes > 0
-                            && ((FileRegion) message).getRemainingBytes() > 0) {
+                    if (( localWrittenBytes > 0 )
+                            && ( ((FileRegion) message).getRemainingBytes() > 0 )) {
                         writtenBytes += localWrittenBytes;
                         setInterestedInWrite(session, true);
                         return false;
@@ -900,7 +923,7 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
             if (req != null) {
                 req.getFuture().setException(e);
             }
-            
+
             IoFilterChain filterChain = session.getFilterChain();
             filterChain.fireExceptionCaught(e);
             return false;
@@ -914,28 +937,28 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
             throws Exception {
         IoBuffer buf = (IoBuffer) req.getMessage();
         int localWrittenBytes = 0;
-        
+
         if (buf.hasRemaining()) {
             int length;
-            
+
             if (hasFragmentation) {
                 length = Math.min(buf.remaining(), maxLength);
             } else {
                 length = buf.remaining();
             }
-            
+
             localWrittenBytes = write(session, buf, length);
         }
 
         session.increaseWrittenBytes(localWrittenBytes, currentTime);
 
-        if (!buf.hasRemaining() || !hasFragmentation && localWrittenBytes != 0) {
+        if (!buf.hasRemaining() || ( !hasFragmentation && ( localWrittenBytes != 0 ) )) {
             // Buffer has been sent, clear the current request.
             int pos = buf.position();
             buf.reset();
-            
+
             fireMessageSent(session, req);
-            
+
             // And set it back to its position
             buf.position(pos);
         }
@@ -947,17 +970,17 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
             throws Exception {
         int localWrittenBytes;
         FileRegion region = (FileRegion) req.getMessage();
-        
+
         if (region.getRemainingBytes() > 0) {
             int length;
-            
+
             if (hasFragmentation) {
                 length = (int) Math.min(region.getRemainingBytes(), maxLength);
             } else {
                 length = (int) Math.min(Integer.MAX_VALUE, region
                         .getRemainingBytes());
             }
-            
+
             localWrittenBytes = transferFile(session, region, length);
             region.update(localWrittenBytes);
         } else {
@@ -966,8 +989,8 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
 
         session.increaseWrittenBytes(localWrittenBytes, currentTime);
 
-        if (region.getRemainingBytes() <= 0 || !hasFragmentation
-                && localWrittenBytes != 0) {
+        if (( region.getRemainingBytes() <= 0 ) || ( !hasFragmentation
+                && ( localWrittenBytes != 0 ) )) {
             fireMessageSent(session, req);
         }
 
@@ -1001,10 +1024,10 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
                     updateTrafficControl(session);
 
                     break;
-    
+
                 case CLOSING:
                     break;
-    
+
                 case OPENING:
                     // Retry later if session is not yet fully initialized.
                     // (In case that Session.suspend??() or session.resume??() is
@@ -1012,14 +1035,14 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
                     // We just put back the session at the end of the queue.
                     trafficControllingSessions.add(session);
                     break;
-    
+
                 default:
                     throw new IllegalStateException(String.valueOf(state));
             }
-            
+
             // As we have handled one session, decrement the number of
             // remaining sessions. The OPENING session will be processed
-            // with the next select(), as the queue size has been decreased, even 
+            // with the next select(), as the queue size has been decreased, even
             // if the session has been pushed at the end of the queue
             queueSize--;
         }
@@ -1029,7 +1052,7 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
      * {@inheritDoc}
      */
     public void updateTrafficControl(S session) {
-        // 
+        //
         try {
             setInterestedInRead(session, !session.isReadSuspended());
         } catch (Exception e) {
@@ -1048,13 +1071,15 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
     }
 
     /**
-     * The main loop. This is the place in charge to poll the Selector, and to 
-     * process the active sessions. It's done in 
+     * The main loop. This is the place in charge to poll the Selector, and to
+     * process the active sessions. It's done in
      * - handle the newly created sessions
-     * - 
+     * -
      */
     private class Processor implements Runnable {
         public void run() {
+            assert (processorRef.get() == this);
+
             int nSessions = 0;
             lastIdleCheckTime = System.currentTimeMillis();
 
@@ -1064,11 +1089,49 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
                     // idle session when we get out of the select every
                     // second. (note : this is a hack to avoid creating
                     // a dedicated thread).
+                    long t0 = System.currentTimeMillis();
                     int selected = select(SELECT_TIMEOUT);
+                    long t1 = System.currentTimeMillis();
+                    long delta = (t1 - t0);
+
+                    if ((selected == 0) && !wakeupCalled.get() && (delta < 100)) {
+                        // Last chance : the select() may have been
+                        // interrupted because we have had an closed channel.
+                        if (isBrokenConnection()) {
+                            LOG.warn("Broken connection");
+
+                            // we can reselect immediately
+                            // set back the flag to false
+                            wakeupCalled.getAndSet(false);
+
+                            continue;
+                        } else {
+                            LOG.warn("Create a new selector. Selected is 0, delta = "
+                                            + (t1 - t0));
+                            // Ok, we are hit by the nasty epoll
+                            // spinning.
+                            // Basically, there is a race condition
+                            // which causes a closing file descriptor not to be
+                            // considered as available as a selected channel, but
+                            // it stopped the select. The next time we will
+                            // call select(), it will exit immediately for the same
+                            // reason, and do so forever, consuming 100%
+                            // CPU.
+                            // We have to destroy the selector, and
+                            // register all the socket on a new one.
+                            registerNewSelector();
+                        }
+
+                        // Set back the flag to false
+                        wakeupCalled.getAndSet(false);
+
+                        // and continue the loop
+                        continue;
+                    }
 
                     // Manage newly created session first
                     nSessions += handleNewSessions();
-                    
+
                     updateTrafficMask();
 
                     // Now, if we have had some incoming or outgoing events,
@@ -1081,22 +1144,33 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
                     // Write the pending requests
                     long currentTime = System.currentTimeMillis();
                     flush(currentTime);
-                    
+
                     // And manage removed sessions
                     nSessions -= removeSessions();
-                    
+
                     // Last, not least, send Idle events to the idle sessions
                     notifyIdleSessions(currentTime);
 
                     // Get a chance to exit the infinite loop if there are no
                     // more sessions on this Processor
                     if (nSessions == 0) {
-                        synchronized (lock) {
-                            if (newSessions.isEmpty() && isSelectorEmpty()) {
-                                processor = null;
-                                break;
-                            }
+                        processorRef.set(null);
+                        
+                        if (newSessions.isEmpty() && isSelectorEmpty()) {
+                            // newSessions.add() precedes startupProcessor
+                            assert (processorRef.get() != this);
+                            break;
                         }
+                        
+                        assert (processorRef.get() != this);
+                        
+                        if (!processorRef.compareAndSet(null, this)) {
+                            // startupProcessor won race, so must exit processor
+                            assert (processorRef.get() != this);
+                            break;
+                        }
+                        
+                        assert (processorRef.get() == this);
                     }
 
                     // Disconnect all sessions immediately if disposal has been
@@ -1105,9 +1179,12 @@ public abstract class AbstractPollingIoProcessor<S extends AbstractIoSession> im
                         for (Iterator<S> i = allSessions(); i.hasNext();) {
                             scheduleRemove(i.next());
                         }
-                        
+
                         wakeup();
                     }
+                } catch (ClosedSelectorException cse) {
+                    // If the selector has been closed, we can exit the loop
+                    break;
                 } catch (Throwable t) {
                     ExceptionMonitor.getInstance().exceptionCaught(t);
 
